/* GCompris - NoteNotations.js
 *
 * SPDX-FileCopyrightText: 2018 Aman Kumar Gupta <gupta2140@gmail.com>
 * SPDX-FileCopyrightText: 2025 Timothée Giet <animtim@gmail.com>
 *
 * Authors:
 *   Beth Hadley <bethmhadley@gmail.com> (GTK+ version)
 *   Johnny Jazeix <jazeix@gmail.com> (Qt Quick port)
 *   Aman Kumar Gupta <gupta2140@gmail.com> (Qt Quick port)
 *   Timothée Giet <animtim@gmail.com> (Refactoring)
 *
 *   SPDX-License-Identifier: GPL-3.0-or-later
 **/

function get() {
    return {
            "TrebleD3": 9,
            "TrebleE3": 8.5,
            "TrebleF#3": 8,
            "TrebleF3": 8,
            "TrebleGb3": 7.5,
            "TrebleG3": 7.5,
            "TrebleG#3": 7.5,
            "TrebleA3": 7,
            "TrebleA#3": 7,
            "TrebleAb3": 7,
            "TrebleB3": 6.5,
            "TrebleBb3": 6.5,
            "TrebleC4": 6,
            "TrebleC#4": 6,
            "TrebleDb4": 5.5,
            "TrebleD4": 5.5,
            "TrebleD#4": 5.5,
            "TrebleEb4": 5,
            "TrebleE4": 5,
            "TrebleF4": 4.5,
            "TrebleF#4": 4.5,
            "TrebleG4": 4,
            "TrebleGb4": 4,
            "TrebleG#4": 4,
            "TrebleAb4": 3.5,
            "TrebleA4": 3.5,
            "TrebleA#4": 3.5,
            "TrebleBb4": 3,
            "TrebleB4": 3,
            "TrebleC5": 2.5,
            "TrebleC#5": 2.5,
            "TrebleDb5": 2,
            "TrebleD5": 2,
            "TrebleD#5": 2,
            "TrebleEb5": 1.5,
            "TrebleE5": 1.5,
            "TrebleF5": 1,
            "TrebleF#5": 1,
            "TrebleG5": 0.5,
            "TrebleGb5": 0.5,
            "TrebleG#5": 0.5,
            "TrebleAb5": 0,
            "TrebleA5": 0,
            "TrebleA#5": 0,
            "TrebleBb5": -0.5,
            "TrebleB5": -0.5,
            "TrebleC6": -1,
            "TrebleC#6": -1,
            "TrebleDb6": -1.5,
            "TrebleD6": -1.5,
            "TrebleD#6": -1.5,
            "TrebleEb6": -2,
            "TrebleE6": -2,
            "TrebleF6": -2.5,
            "BassF1": 9,
            "BassF#1": 9,
            "BassGb1": 8.5,
            "BassG1": 8.5,
            "BassG#1": 8.5,
            "BassAb1": 8,
            "BassA1": 8,
            "BassA#1": 8,
            "BassBb1": 7.5,
            "BassB1": 7.5,
            "BassC2": 7,
            "BassC#2": 7,
            "BassDb2": 6.5,
            "BassD2": 6.5,
            "BassD#2": 6.5,
            "BassEb2": 6,
            "BassE2": 6,
            "BassF2": 5.5,
            "BassF#2": 5.5,
            "BassGb2": 5,
            "BassG2": 5,
            "BassG#2": 5,
            "BassAb2": 4.5,
            "BassA2": 4.5,
            "BassA#2": 4.5,
            "BassBb2": 4,
            "BassB2": 4,
            "BassC3": 3.5,
            "BassC#3": 3.5,
            "BassDb3": 3,
            "BassD3": 3,
            "BassD#3": 3,
            "BassEb3": 2.5,
            "BassE3": 2.5,
            "BassF3": 2,
            "BassF#3": 2,
            "BassG3": 1.5,
            "BassGb3": 1.5,
            "BassG#3": 1.5,
            "BassAb3": 1,
            "BassA3": 1,
            "BassA#3": 1,
            "BassBb3": 0.5,
            "BassB3": 0.5,
            "BassC4": 0,
            "BassC#4": 0,
            "BassD4": -0.5,
            "BassD#4": -0.5,
            "BassDb4": -0.5,
            "BassE4": -1,
            "BassEb4": -1,
            "BassF4": -1.5,
            "BassF#4": -1.5,
            "BassG4": -2,
            "BassGb4": -2,
            "BassA4": -2.5,
            "BassB4": -3,
            "wholeRest": 5,
            "halfRest": 5,
            "quarterRest": 5,
            "eighthRest": 5
        }
}
